@@ -16,6 +16,7 @@ The classes include:
 import numpy as np
 import inspect
 from parmoo.structs import SurrogateOptimizer, AcquisitionFunction
+from parmoo.util import xerror
 
 
 class LBFGSB(SurrogateOptimizer):
@@ -29,10 +30,10 @@ class LBFGSB(SurrogateOptimizer):
 
     # Slots for the LBFGSB class
     __slots__ = ['n', 'bounds', 'acquisitions', 'budget', 'constraints',
-                 'objectives', 'gradients', 'lagrangian']
+                 'objectives', 'gradients', 'penalty_func', 'restarts']
 
     def __init__(self, o, lb, ub, hyperparams):
-        """ Constructor for the LocalGPS class.
+        """ Constructor for the LBFGSB class.
 
         Args:
             o (int): The number of objectives.
@@ -46,22 +47,35 @@ class LBFGSB(SurrogateOptimizer):
 
             hyperparams (dict): A dictionary of hyperparameters for the
                 optimization procedure. It may contain the following:
-                 * opt_budget (int): The evaluation budget (default: 10,000).
+                 * opt_budget (int): The evaluation budget per solve
+                   (default: 1000).
+                 * opt_restarts (int): Number of multisolve restarts per
+                   scalarization (default: n+1).
 
         Returns:
             SurrogateOptimizer: A new SurrogateOptimizer object.
 
         """
 
-        from parmoo.util import xerror
-
         # Check inputs
-        xerror(o, lb, ub, hyperparams)
+        xerror(o=o, lb=lb, ub=ub, hyperparams=hyperparams)
         self.n = lb.size
         self.bounds = np.zeros((self.n, 2))
         self.bounds[:, 0] = lb
         self.bounds[:, 1] = ub
         # Check that the contents of hyperparams is legal
+        if 'opt_restarts' in hyperparams:
+            if isinstance(hyperparams['opt_restarts'], int):
+                if hyperparams['opt_restarts'] < 1:
+                    raise ValueError("hyperparams['opt_restarts'] "
+                                     "must be positive")
+                else:
+                    self.restarts = hyperparams['opt_restarts']
+            else:
+                raise TypeError("hyperparams['opt_restarts'] "
+                                 "must be an integer")
+        else:
+            self.restarts = self.n + 1
         if 'opt_budget' in hyperparams:
             if isinstance(hyperparams['opt_budget'], int):
                 if hyperparams['opt_budget'] < 1:
@@ -70,10 +84,10 @@ class LBFGSB(SurrogateOptimizer):
                 else:
                     self.budget = hyperparams['opt_budget']
             else:
-                raise ValueError("hyperparams['opt_budget'] "
+                raise TypeError("hyperparams['opt_budget'] "
                                  "must be an integer")
         else:
-            self.budget = 10000
+            self.budget = 1000
         self.acquisitions = []
         return
 
@@ -94,7 +108,7 @@ class LBFGSB(SurrogateOptimizer):
                 # Add obj_func to the problem
                 self.objectives = obj_func
         else:
-            raise ValueError("obj_func() must be callable")
+            raise TypeError("obj_func() must be callable")
         return
 
     def setReset(self, reset):
@@ -106,36 +120,36 @@ class LBFGSB(SurrogateOptimizer):
 
         return
 
-    def setLagrangian(self, lagrangian, grad_func):
+    def setPenalty(self, penalty_func, grad_func):
         """ Add a matrix-valued gradient function for obj_func.
 
         Args:
-            lagrangian (function): A vector-valued augmented Lagrangian
+            penalty_func (function): A vector-valued penalized objective
                 that incorporates a penalty for violating constraints.
 
             grad_func (function): A matrix-valued function that can be
-                evaluated to obtain the Jacobian matrix for the Lagrangian.
+                evaluated to obtain the Jacobian matrix for obj_func.
 
         """
 
         # Check whether grad_func() has an appropriate signature
         if callable(grad_func):
             if len(inspect.signature(grad_func).parameters) != 1:
-                raise ValueError("grad_func() must accept exactly one input")
+                raise ValueError("grad_func must accept exactly one input")
             else:
                 # Add grad_func to the problem
                 self.gradients = grad_func
         else:
-            raise ValueError("grad_func() must be callable")
-        # Check whether lagrangian() has an appropriate signature
-        if callable(lagrangian):
-            if len(inspect.signature(lagrangian).parameters) != 1:
-                raise ValueError("lagrangian() must accept exactly one input")
+            raise TypeError("grad_func() must be callable")
+        # Check whether penalty_func() has an appropriate signature
+        if callable(penalty_func):
+            if len(inspect.signature(penalty_func).parameters) != 1:
+                raise ValueError("penalty_func must accept exactly one input")
             else:
                 # Add Lagrangian to the problem
-                self.lagrangian = lagrangian
+                self.penalty_func = penalty_func
         else:
-            raise ValueError("lagrangian() must be callable")
+            raise TypeError("penalty_func must be callable")
         return
 
     def setConstraints(self, constraint_func):
@@ -158,7 +172,7 @@ class LBFGSB(SurrogateOptimizer):
                 # Add constraint_func to the problem
                 self.constraints = constraint_func
         else:
-            raise ValueError("constraint_func() must be callable")
+            raise TypeError("constraint_func() must be callable")
         return
 
     def addAcquisition(self, *args):
@@ -173,7 +187,7 @@ class LBFGSB(SurrogateOptimizer):
 
         # Check for illegal inputs
         if not all([isinstance(arg, AcquisitionFunction) for arg in args]):
-            raise ValueError("Args must be instances of AcquisitionFunction")
+            raise TypeError("Args must be instances of AcquisitionFunction")
         # Append all arguments to the acquisitions list
         for arg in args:
             self.acquisitions.append(arg)
@@ -183,8 +197,8 @@ class LBFGSB(SurrogateOptimizer):
         """ Solve the surrogate problem using L-BFGS-B.
 
         Args:
-            x (np.ndarray): A 2d array containing a list of feasible
-                design points used to warm start the search.
+            x (np.ndarray): A 2d array containing a list of design points
+                used to warm start the search.
 
         Returns:
             np.ndarray: A 2d numpy.ndarray of potentially efficient design
@@ -202,7 +216,7 @@ class LBFGSB(SurrogateOptimizer):
                 raise ValueError("The rows of x must match the number " +
                                  "of acquisition functions")
         else:
-            raise ValueError("x must be a numpy array")
+            raise TypeError("x must be a numpy array")
         # Check that x is feasible.
         for xj in x:
             if np.any(xj[:] < self.bounds[:, 0]) or \
@@ -210,25 +224,47 @@ class LBFGSB(SurrogateOptimizer):
                 raise ValueError("some of starting points (x) are infeasible")
         # Initialize an empty list of results
         result = []
-        # Calculate budget per call
-        budget_per_call = self.budget / len(self.acquisitions)
         # For each acqusisition function
         for j, acquisition in enumerate(self.acquisitions):
 
             # Define the scalarized wrapper functions
             def scalar_f(x, *args):
-                return acquisition.scalarize(self.lagrangian(x))
+                return acquisition.scalarize(self.penalty_func(x))
 
             def scalar_g(x, *args):
-                return acquisition.scalarizeGrad(self.lagrangian(x),
+                return acquisition.scalarizeGrad(self.penalty_func(x),
                                                  self.gradients(x))
 
-            # Get the solution
-            res = optimize.minimize(scalar_f, x[j, :], method='L-BFGS-B',
-                                    jac=scalar_g, bounds=self.bounds,
-                                    options={'maxiter': budget_per_call})
+            # Get the solution via multistart solve
+            soln = x[j, :].copy()
+            for i in range(self.restarts):
+                if i == 0:
+                    # Use center point to warm-start first start
+                    x0 = x[j, :].copy()
+                elif i == 1:
+                    # Use predicted gradient step to warm-start second start
+                    x0 = x[j, :].copy()
+                    gg = scalar_g(x0)
+                    for ii in range(self.n):
+                        if gg[ii] < 0:
+                            x0[ii] = self.bounds[ii, 1]
+                        elif gg[ii] > 0:
+                            x0[ii] = self.bounds[ii, 0]
+                else:
+                    # Random starting point within bounds for all other starts
+                    x0 = (np.random.random_sample(self.n) *
+                          (self.bounds[:, 1] - self.bounds[:, 0]) +
+                          self.bounds[:, 0])
+
+                # Solve the problem globally within bound constraints
+                res = optimize.minimize(scalar_f, x0, method='L-BFGS-B',
+                                        jac=scalar_g, bounds=self.bounds,
+                                        options={'maxiter': self.budget})
+                if scalar_f(res['x']) < scalar_f(soln):
+                    soln = res['x']
+
             # Append the found minima to the results list
-            result.append(res['x'])
+            result.append(soln)
         return np.asarray(result)
 
 
@@ -243,10 +279,11 @@ class TR_LBFGSB(SurrogateOptimizer):
 
     # Slots for the LBFGSB class
     __slots__ = ['n', 'bounds', 'acquisitions', 'budget', 'constraints',
-                 'objectives', 'gradients', 'lagrangian', 'resetObjectives']
+                 'objectives', 'gradients', 'penalty_func', 'resetObjectives',
+                 'restarts']
 
     def __init__(self, o, lb, ub, hyperparams):
-        """ Constructor for the LocalGPS class.
+        """ Constructor for the TR_LBFGSB class.
 
         Args:
             o (int): The number of objectives.
@@ -260,17 +297,18 @@ class TR_LBFGSB(SurrogateOptimizer):
 
             hyperparams (dict): A dictionary of hyperparameters for the
                 optimization procedure. It may contain the following:
-                 * opt_budget (int): The evaluation budget (default: 10,000).
+                 * opt_budget (int): The evaluation budget per solve
+                   (default: 1000).
+                 * opt_restarts (int): Number of multisolve restarts per
+                   scalarization (default: 2).
 
         Returns:
             SurrogateOptimizer: A new SurrogateOptimizer object.
 
         """
 
-        from parmoo.util import xerror
-
         # Check inputs
-        xerror(o, lb, ub, hyperparams)
+        xerror(o=o, lb=lb, ub=ub, hyperparams=hyperparams)
         self.n = lb.size
         self.bounds = np.zeros((self.n, 2))
         self.bounds[:, 0] = lb
@@ -284,10 +322,23 @@ class TR_LBFGSB(SurrogateOptimizer):
                 else:
                     self.budget = hyperparams['opt_budget']
             else:
-                raise ValueError("hyperparams['opt_budget'] "
+                raise TypeError("hyperparams['opt_budget'] "
                                  "must be an integer")
         else:
-            self.budget = 10000
+            self.budget = 1000
+        # Check that the contents of hyperparams is legal
+        if 'opt_restarts' in hyperparams:
+            if isinstance(hyperparams['opt_restarts'], int):
+                if hyperparams['opt_restarts'] < 1:
+                    raise ValueError("hyperparams['opt_restarts'] "
+                                     "must be positive")
+                else:
+                    self.restarts = hyperparams['opt_restarts']
+            else:
+                raise TypeError("hyperparams['opt_restarts'] "
+                                 "must be an integer")
+        else:
+            self.restarts = 2
         self.acquisitions = []
         return
 
@@ -308,7 +359,7 @@ class TR_LBFGSB(SurrogateOptimizer):
                 # Add obj_func to the problem
                 self.objectives = obj_func
         else:
-            raise ValueError("obj_func() must be callable")
+            raise TypeError("obj_func() must be callable")
         return
 
     def setReset(self, reset):
@@ -329,18 +380,18 @@ class TR_LBFGSB(SurrogateOptimizer):
                 # Add obj_func to the problem
                 self.resetObjectives = reset
         else:
-            raise ValueError("reset() must be callable")
+            raise TypeError("reset() must be callable")
         return
 
-    def setLagrangian(self, lagrangian, grad_func):
+    def setPenalty(self, penalty_func, grad_func):
         """ Add a matrix-valued gradient function for obj_func.
 
         Args:
-            lagrangian (function): A vector-valued augmented Lagrangian
+            penalty_func (function): A vector-valued penalized objective
                 that incorporates a penalty for violating constraints.
 
             grad_func (function): A matrix-valued function that can be
-                evaluated to obtain the Jacobian matrix for the Lagrangian.
+                evaluated to obtain the Jacobian matrix for obj_func.
 
         """
 
@@ -352,16 +403,16 @@ class TR_LBFGSB(SurrogateOptimizer):
                 # Add grad_func to the problem
                 self.gradients = grad_func
         else:
-            raise ValueError("grad_func() must be callable")
-        # Check whether lagrangian() has an appropriate signature
-        if callable(lagrangian):
-            if len(inspect.signature(lagrangian).parameters) != 1:
-                raise ValueError("lagrangian() must accept exactly one input")
+            raise TypeError("grad_func() must be callable")
+        # Check whether penalty_func() has an appropriate signature
+        if callable(penalty_func):
+            if len(inspect.signature(penalty_func).parameters) != 1:
+                raise ValueError("penalty_func must accept exactly one input")
             else:
                 # Add Lagrangian to the problem
-                self.lagrangian = lagrangian
+                self.penalty_func = penalty_func
         else:
-            raise ValueError("lagrangian() must be callable")
+            raise TypeError("penalty_func must be callable")
         return
 
     def setConstraints(self, constraint_func):
@@ -384,7 +435,7 @@ class TR_LBFGSB(SurrogateOptimizer):
                 # Add constraint_func to the problem
                 self.constraints = constraint_func
         else:
-            raise ValueError("constraint_func() must be callable")
+            raise TypeError("constraint_func() must be callable")
         return
 
     def addAcquisition(self, *args):
@@ -399,7 +450,7 @@ class TR_LBFGSB(SurrogateOptimizer):
 
         # Check for illegal inputs
         if not all([isinstance(arg, AcquisitionFunction) for arg in args]):
-            raise ValueError("Args must be instances of AcquisitionFunction")
+            raise TypeError("Args must be instances of AcquisitionFunction")
         # Append all arguments to the acquisitions list
         for arg in args:
             self.acquisitions.append(arg)
@@ -409,8 +460,8 @@ class TR_LBFGSB(SurrogateOptimizer):
         """ Solve the surrogate problem using L-BFGS-B.
 
         Args:
-            x (np.ndarray): A 2d array containing a list of feasible
-                design points used to warm start the search.
+            x (np.ndarray): A 2d array containing a list of design points
+                used to warm start the search.
 
         Returns:
             np.ndarray: A 2d numpy.ndarray of potentially efficient design
@@ -428,7 +479,7 @@ class TR_LBFGSB(SurrogateOptimizer):
                 raise ValueError("The rows of x must match the number " +
                                  "of acquisition functions")
         else:
-            raise ValueError("x must be a numpy array")
+            raise TypeError("x must be a numpy array")
         # Check that x is feasible.
         for xj in x:
             if np.any(xj[:] < self.bounds[:, 0]) or \
@@ -436,17 +487,15 @@ class TR_LBFGSB(SurrogateOptimizer):
                 raise ValueError("some of starting points (x) are infeasible")
         # Initialize an empty list of results
         result = []
-        # Calculate budget per call
-        budget_per_call = self.budget / len(self.acquisitions)
         # For each acqusisition function
         for j, acquisition in enumerate(self.acquisitions):
 
             # Define the scalarized wrapper functions
             def scalar_f(x, *args):
-                return acquisition.scalarize(self.lagrangian(x))
+                return acquisition.scalarize(self.penalty_func(x))
 
             def scalar_g(x, *args):
-                return acquisition.scalarizeGrad(self.lagrangian(x),
+                return acquisition.scalarizeGrad(self.penalty_func(x),
                                                  self.gradients(x))
 
             # Create a new trust region
@@ -455,10 +504,33 @@ class TR_LBFGSB(SurrogateOptimizer):
             for i in range(self.n):
                 bounds[i, 0] = max(self.bounds[i, 0], x[j, i] - rad)
                 bounds[i, 1] = min(self.bounds[i, 1], x[j, i] + rad)
-            # Get the solution
-            res = optimize.minimize(scalar_f, x[j, :], method='L-BFGS-B',
-                                    jac=scalar_g, bounds=bounds,
-                                    options={'maxiter': budget_per_call})
+
+            # Get the solution via multistart solve
+            soln = x[j, :].copy()
+            for i in range(self.restarts):
+                if i == 0:
+                    # Use center point to warm-start first start
+                    x0 = x[j, :].copy()
+                elif i == 1:
+                    # Use predicted gradient step to warm-start second start
+                    x0 = x[j, :].copy()
+                    gg = scalar_g(x0)
+                    for ii in range(self.n):
+                        if gg[ii] < 0:
+                            x0[ii] = bounds[ii, 1]
+                        elif gg[ii] > 0:
+                            x0[ii] = bounds[ii, 0]
+                else:
+                    # Random starting point within bounds for all other starts
+                    x0 = (np.random.random_sample(self.n) *
+                          (bounds[:, 1] - bounds[:, 0]) + bounds[:, 0])
+
+                # Solve the problem within the local trust region
+                res = optimize.minimize(scalar_f, x0, method='L-BFGS-B',
+                                        jac=scalar_g, bounds=bounds,
+                                        options={'maxiter': self.budget})
+                if scalar_f(res['x']) < scalar_f(soln):
+                    soln = res['x']
             # Append the found minima to the results list
-            result.append(res['x'])
+            result.append(soln)
         return np.asarray(result)
