@@ -13,7 +13,7 @@ The classes include:
 
 """
 
-from abc import ABC
+from abc import ABC, abstractmethod
 
 
 class AcquisitionFunction(ABC):
@@ -28,6 +28,7 @@ class AcquisitionFunction(ABC):
 
     """
 
+    @abstractmethod
     def __init__(self, o, lb, ub, hyperparams):
         """ Constructor for the AcquisitionFunction class.
 
@@ -48,8 +49,7 @@ class AcquisitionFunction(ABC):
 
         """
 
-        raise NotImplementedError("This class method has not been implemented")
-
+    @abstractmethod
     def setTarget(self, data, constraint_func, history):
         """ Set a new target value or region for the AcquisitionFunction.
 
@@ -82,8 +82,7 @@ class AcquisitionFunction(ABC):
 
         """
 
-        raise NotImplementedError("This class method has not been implemented")
-
+    @abstractmethod
     def scalarize(self, f_vals):
         """ Scalarize a vector-valued function using the AcquisitionFunction.
 
@@ -96,8 +95,6 @@ class AcquisitionFunction(ABC):
 
         """
 
-        raise NotImplementedError("This class method has not been implemented")
-
     def scalarizeGrad(self, f_vals, g_vals):
         """ Scalarize a Jacobian of gradients using the current weights.
 
@@ -157,6 +154,7 @@ class GlobalSearch(ABC):
 
     """
 
+    @abstractmethod
     def __init__(self, o, lb, ub, hyperparams):
         """ Constructor for the GlobalSearch class.
 
@@ -178,8 +176,7 @@ class GlobalSearch(ABC):
 
         """
 
-        raise NotImplementedError("This class method has not been implemented")
-
+    @abstractmethod
     def startSearch(self, lb, ub):
         """ Begin a new global search.
 
@@ -196,8 +193,6 @@ class GlobalSearch(ABC):
 
         """
 
-        raise NotImplementedError("This class method has not been implemented")
-
     def resumeSearch(self):
         """ Resume a global search.
 
@@ -255,6 +250,7 @@ class SurrogateFunction(ABC):
 
     """
 
+    @abstractmethod
     def __init__(self, m, lb, ub, hyperparams):
         """ Constructor for the SurrogateFunction class.
 
@@ -281,8 +277,7 @@ class SurrogateFunction(ABC):
 
         """
 
-        raise NotImplementedError("This class method has not been implemented")
-
+    @abstractmethod
     def fit(self, x, f):
         """ Fit a new surrogate to the given data.
 
@@ -295,8 +290,7 @@ class SurrogateFunction(ABC):
 
         """
 
-        raise NotImplementedError("This class method has not been implemented")
-
+    @abstractmethod
     def update(self, x, f):
         """ Update an existing surrogate model using new data.
 
@@ -310,8 +304,7 @@ class SurrogateFunction(ABC):
 
         """
 
-        raise NotImplementedError("This class method has not been implemented")
-
+    @abstractmethod
     def setCenter(self, center):
         """ Set the center for the fit, if this is a local method.
 
@@ -321,8 +314,7 @@ class SurrogateFunction(ABC):
 
         """
 
-        raise NotImplementedError("This class method has not been implemented")
-
+    @abstractmethod
     def evaluate(self, x):
         """ Evaluate the surrogate at a design point.
 
@@ -336,8 +328,6 @@ class SurrogateFunction(ABC):
 
         """
 
-        raise NotImplementedError("This class method has not been implemented")
-
     def gradient(self, x):
         """ Evaluate the gradient of the surrogate at a design point.
 
@@ -353,6 +343,7 @@ class SurrogateFunction(ABC):
 
         raise NotImplementedError("This class method has not been implemented")
 
+    @abstractmethod
     def improve(self, x, global_imrpov):
         """ Suggests a design to evaluate to improve the surrogate near x.
 
@@ -369,8 +360,6 @@ class SurrogateFunction(ABC):
 
         """
 
-        raise NotImplementedError("This class method has not been implemented")
-
     def save(self, filename):
         """ Save important data from this class so that it can be reloaded.
 
@@ -417,6 +406,7 @@ class SurrogateOptimizer(ABC):
 
     """
 
+    @abstractmethod
     def __init__(self, o, lb, ub, hyperparams):
         """ Constructor for the SurrogateOptimizer class.
 
@@ -437,8 +427,7 @@ class SurrogateOptimizer(ABC):
 
         """
 
-        raise NotImplementedError("This class method has not been implemented")
-
+    @abstractmethod
     def setObjective(self, obj_func):
         """ Add a vector-valued objective function that will be solved.
 
@@ -448,8 +437,6 @@ class SurrogateOptimizer(ABC):
 
         """
 
-        raise NotImplementedError("This class method has not been implemented")
-
     def setGradient(self, grad_func):
         """ Add a matrix-valued gradient function for obj_func.
 
@@ -461,6 +448,7 @@ class SurrogateOptimizer(ABC):
 
         raise NotImplementedError("This class method has not been implemented")
 
+    @abstractmethod
     def setConstraints(self, constraint_func):
         """ Add a constraint function that will be satisfied.
 
@@ -472,8 +460,7 @@ class SurrogateOptimizer(ABC):
 
         """
 
-        raise NotImplementedError("This class method has not been implemented")
-
+    @abstractmethod
     def addAcquisition(self, *args):
         """ Add an acquisition function for the surrogate optimizer.
 
@@ -484,8 +471,7 @@ class SurrogateOptimizer(ABC):
 
         """
 
-        raise NotImplementedError("This class method has not been implemented")
-
+    @abstractmethod
     def setReset(self, reset):
         """ Add a reset function for resetting surrogate updates.
 
@@ -496,8 +482,7 @@ class SurrogateOptimizer(ABC):
 
         """
 
-        raise NotImplementedError("This class method has not been implemented")
-
+    @abstractmethod
     def solve(self, x):
         """ Solve the surrogate problem.
 
@@ -511,8 +496,6 @@ class SurrogateOptimizer(ABC):
 
         """
 
-        raise NotImplementedError("This class method has not been implemented")
-
     def save(self, filename):
         """ Save important data from this class so that it can be reloaded.
 
