@@ -13,6 +13,8 @@ The classes include:
 import numpy as np
 from parmoo.structs import SurrogateFunction
 from scipy.spatial.distance import cdist
+from scipy.stats import tstd
+from parmoo.util import xerror
 
 
 class GaussRBF(SurrogateFunction):
@@ -52,10 +54,8 @@ class GaussRBF(SurrogateFunction):
 
         """
 
-        from parmoo.util import xerror
-
         # Check inputs
-        xerror(m, lb, ub, hyperparams)
+        xerror(o=m, lb=lb, ub=ub, hyperparams=hyperparams)
         # Initialize problem dimensions
         self.m = m
         self.lb = lb
@@ -67,7 +67,7 @@ class GaussRBF(SurrogateFunction):
         self.f_vals = np.zeros((0, self.m))
         self.weights = np.zeros((0, 0))
         # Check for the 'nugget' optional value in hyperparams
-        if 'nugget' in hyperparams.keys():
+        if 'nugget' in hyperparams:
             if isinstance(hyperparams['nugget'], float):
                 self.nugget = hyperparams['nugget']
                 if self.nugget < 0.0:
@@ -78,7 +78,7 @@ class GaussRBF(SurrogateFunction):
                                  + " value")
         else:
             self.nugget = 0.0
-        if 'des_tols' in hyperparams.keys():
+        if 'des_tols' in hyperparams:
             if isinstance(hyperparams['des_tols'], np.ndarray):
                 if hyperparams['des_tols'].size == self.n:
                     if np.all(hyperparams['des_tols'] > 0.0):
@@ -115,14 +115,14 @@ class GaussRBF(SurrogateFunction):
 
         # Check that the x and f values are legal
         if not isinstance(x, np.ndarray):
-            raise ValueError("x must be a numpy array")
+            raise TypeError("x must be a numpy array")
         if isinstance(f, np.ndarray):
             if f.shape[0] == 0:
                 raise ValueError("no data provided")
             if self.m != np.size(f[0, :]):
                 raise ValueError("each row of f must have length m")
         else:
-            raise ValueError("f must be a numpy array")
+            raise TypeError("f must be a numpy array")
         if x.shape[0] != f.shape[0]:
             raise ValueError("x and f must have equal lengths")
         # Initialize the internal database with x and f
@@ -171,14 +171,14 @@ class GaussRBF(SurrogateFunction):
 
         # Check that the x and f values are legal
         if not isinstance(x, np.ndarray):
-            raise ValueError("x must be a numpy array")
+            raise TypeError("x must be a numpy array")
         if isinstance(f, np.ndarray):
             if f.shape[0] == 0:
                 return    # No new data, do nothing
             if self.m != np.size(f[0, :]):
                 raise ValueError("each row of f must have length m")
         else:
-            raise ValueError("f must be a numpy array")
+            raise TypeError("f must be a numpy array")
         if x.shape[0] != f.shape[0]:
             raise ValueError("x and f must have equal lengths")
         # Update the internal database with x and f
@@ -238,7 +238,7 @@ class GaussRBF(SurrogateFunction):
 
         # Check that the x is legal
         if not isinstance(x, np.ndarray):
-            raise ValueError("x must be a numpy array")
+            raise TypeError("x must be a numpy array")
         else:
             if x.size != self.n:
                 raise ValueError("x must have length n")
@@ -264,7 +264,7 @@ class GaussRBF(SurrogateFunction):
 
         # Check that the x is legal
         if not isinstance(x, np.ndarray):
-            raise ValueError("x must be a numpy array")
+            raise TypeError("x must be a numpy array")
         else:
             if x.size != self.n:
                 raise ValueError("x must have length n")
@@ -296,7 +296,7 @@ class GaussRBF(SurrogateFunction):
 
         # Check that the x is legal
         if not isinstance(x, np.ndarray):
-            raise ValueError("x must be a numpy array")
+            raise TypeError("x must be a numpy array")
         else:
             if x.size != self.n:
                 raise ValueError("x must have length n")
@@ -315,22 +315,26 @@ class GaussRBF(SurrogateFunction):
                 x_new[:] = self.lb[:] + (np.random.random(self.n)
                                          * (self.ub[:] - self.lb[:]))
         else:
-            # Find the n+1 closest points to x in the current database.
-            dists = np.asarray([np.amax(np.abs(x - xj) / self.eps)
+            # Find the n+1 closest points to x in the current database
+            diffs = np.asarray([np.abs(x - xj) / self.eps
                                 for xj in self.x_vals])
+            dists = np.asarray([np.amax(dj) for dj in diffs])
             inds = np.argsort(dists)
-            if dists[inds[self.n]] >= 1.5:
-                # Uniformly sample within the box [x - rad, x + rad].
-                rad = np.abs(x - self.x_vals[self.n])
-                xn = self.x_vals[self.n]
-                x_new = np.fmin(np.fmax(2.0 * np.random.random(self.n)
-                                        * rad[:] + (x - np.abs(xn)),
-                                        self.lb), self.ub)
+            diffs = diffs[inds]
+            if dists[inds[self.n]] > 1.5:
+                # Calculate the normalized sample std dev along each axis
+                stddev = np.asarray(tstd(diffs[:self.n+1], axis=0))
+                stddev[:] = np.maximum(stddev, np.ones(self.n))
+                stddev[:] = stddev[:] / np.amin(stddev)
+                # Sample within B(x, dists[inds[self.n]] / stddev)
+                rad = (dists[inds[self.n]] * self.eps) / stddev
+                x_new = np.fmin(np.fmax(2.0 * (np.random.random(self.n) - 0.5)
+                                        * rad[:] + x, self.lb), self.ub)
                 while any([np.all(np.abs(x_new - xj) < self.eps)
                            for xj in self.x_vals]):
-                    x_new = np.fmin(np.fmax(2.0 * np.random.random(self.n)
-                                            * rad[:] + (x - np.abs(xn)),
-                                            self.lb), self.ub)
+                    x_new = np.fmin(np.fmax(2.0 *
+                                            (np.random.random(self.n) - 0.5)
+                                            * rad[:] + x, self.lb), self.ub)
             else:
                 # If the n+1st nearest point is too close, use global_improv.
                 x_new[:] = self.lb[:] + np.random.random(self.n) \
@@ -437,10 +441,8 @@ class LocalGaussRBF(SurrogateFunction):
 
         """
 
-        from parmoo.util import xerror
-
         # Check inputs
-        xerror(m, lb, ub, hyperparams)
+        xerror(o=m, lb=lb, ub=ub, hyperparams=hyperparams)
         # Initialize problem dimensions
         self.m = m
         self.lb = lb
@@ -455,7 +457,7 @@ class LocalGaussRBF(SurrogateFunction):
         self.tr_center = np.zeros(0)
         self.loc_inds = []
         # Check for the 'nugget' optional value in hyperparams
-        if 'nugget' in hyperparams.keys():
+        if 'nugget' in hyperparams:
             if isinstance(hyperparams['nugget'], float):
                 self.nugget = hyperparams['nugget']
                 if self.nugget < 0.0:
@@ -467,7 +469,7 @@ class LocalGaussRBF(SurrogateFunction):
         else:
             self.nugget = 0.0
         # Check for the 'n_loc' optional value in hyperparams
-        if 'n_loc' in hyperparams.keys():
+        if 'n_loc' in hyperparams:
             if isinstance(hyperparams['n_loc'], int):
                 self.n_loc = hyperparams['n_loc']
                 if self.n_loc < self.n + 1:
@@ -479,7 +481,7 @@ class LocalGaussRBF(SurrogateFunction):
         else:
             self.n_loc = self.n + 1
         # Check for 'des_tols' optional key in hyperparms
-        if 'des_tols' in hyperparams.keys():
+        if 'des_tols' in hyperparams:
             if isinstance(hyperparams['des_tols'], np.ndarray):
                 if hyperparams['des_tols'].size == self.n:
                     if np.all(hyperparams['des_tols'] > 0.0):
@@ -516,14 +518,14 @@ class LocalGaussRBF(SurrogateFunction):
 
         # Check that the x and f values are legal
         if not isinstance(x, np.ndarray):
-            raise ValueError("x must be a numpy array")
+            raise TypeError("x must be a numpy array")
         if isinstance(f, np.ndarray):
             if f.shape[0] == 0:
                 raise ValueError("no data provided")
             if self.m != np.size(f[0, :]):
                 raise ValueError("each row of f must have length m")
         else:
-            raise ValueError("f must be a numpy array")
+            raise TypeError("f must be a numpy array")
         if x.shape[0] != f.shape[0]:
             raise ValueError("x and f must have equal lengths")
         # Initialize the internal database with x and f
@@ -548,14 +550,14 @@ class LocalGaussRBF(SurrogateFunction):
 
         # Check that the x and f values are legal
         if not isinstance(x, np.ndarray):
-            raise ValueError("x must be a numpy array")
+            raise TypeError("x must be a numpy array")
         if isinstance(f, np.ndarray):
             if f.shape[0] == 0:
                 return    # No new data, do nothing
             if self.m != np.size(f[0, :]):
                 raise ValueError("each row of f must have length m")
         else:
-            raise ValueError("f must be a numpy array")
+            raise TypeError("f must be a numpy array")
         if x.shape[0] != f.shape[0]:
             raise ValueError("x and f must have equal lengths")
         # Update the internal database with x and f
@@ -581,7 +583,7 @@ class LocalGaussRBF(SurrogateFunction):
 
         # Check that the center is legal
         if not isinstance(center, np.ndarray):
-            raise ValueError("center must be a numpy array")
+            raise TypeError("center must be a numpy array")
         else:
             if center.size != self.n:
                 raise ValueError("center must have length n")
@@ -639,7 +641,7 @@ class LocalGaussRBF(SurrogateFunction):
 
         # Check that the x is legal
         if not isinstance(x, np.ndarray):
-            raise ValueError("x must be a numpy array")
+            raise TypeError("x must be a numpy array")
         else:
             if x.size != self.n:
                 raise ValueError("x must have length n")
@@ -666,7 +668,7 @@ class LocalGaussRBF(SurrogateFunction):
 
         # Check that the x is legal
         if not isinstance(x, np.ndarray):
-            raise ValueError("x must be a numpy array")
+            raise TypeError("x must be a numpy array")
         else:
             if x.size != self.n:
                 raise ValueError("x must have length n")
@@ -699,14 +701,14 @@ class LocalGaussRBF(SurrogateFunction):
 
         # Check that the x is legal
         if not isinstance(x, np.ndarray):
-            raise ValueError("x must be a numpy array")
+            raise TypeError("x must be a numpy array")
         else:
             if x.size != self.n:
                 raise ValueError("x must have length n")
             elif (np.any(x < self.lb - self.eps) or
                   np.any(x > self.ub + self.eps)):
                 raise ValueError("x cannot be infeasible")
-        # Allocate the output array.
+        # Allocate the output array
         x_new = np.zeros(self.n)
         if global_improv:
             # If global improvement has been specified, randomly select a
@@ -719,27 +721,30 @@ class LocalGaussRBF(SurrogateFunction):
                                          * (self.ub[:] - self.lb[:]))
         else:
             # Find the n_loc closest points to x in the current database
-            dists = np.asarray([np.amax(np.abs(x - xj) / self.eps)
+            diffs = np.asarray([np.abs(x - xj) / self.eps
                                 for xj in self.x_vals])
+            dists = np.asarray([np.amax(dj) for dj in diffs])
             inds = np.argsort(dists)
+            diffs = diffs[inds]
             if dists[inds[self.n_loc - 1]] > 1.5:
-                # Uniformly sample within B(x, dists[n_loc]).
-                xn = self.x_vals[self.n_loc - 1]
-                rad = np.abs(x - self.x_vals[self.n_loc - 1])
-                x_new = np.fmin(np.fmax(2.0 * np.random.random(self.n)
-                                        * rad[:] + (x - np.abs(xn)),
-                                        self.lb), self.ub)
+                # Calculate the normalized sample std dev along each axis
+                stddev = np.asarray(tstd(diffs[:self.n_loc], axis=0))
+                stddev[:] = np.maximum(stddev, np.ones(self.n))
+                stddev[:] = stddev[:] / np.amin(stddev)
+                # Sample within B(x, dists[inds[n_loc - 1]] / stddev)
+                rad = (dists[inds[self.n_loc - 1]] * self.eps) / stddev
+                x_new = np.fmin(np.fmax(2.0 * (np.random.random(self.n) - 0.5)
+                                        * rad[:] + x, self.lb), self.ub)
                 while any([np.all(np.abs(x_new - xj) < self.eps)
                            for xj in self.x_vals]):
-                    x_new = np.fmin(np.fmax(2.0 * rad[:] *
-                                            np.random.random(self.n) +
-                                            (x - np.abs(xn)),
-                                            self.lb), self.ub)
+                    x_new = np.fmin(np.fmax(2.0 *
+                                            (np.random.random(self.n) - 0.5)
+                                            * rad[:] + x, self.lb), self.ub)
             else:
                 # If the n_loc nearest point is too close, use global_improv
                 x_new[:] = self.lb[:] + np.random.random(self.n) \
                            * (self.ub[:] - self.lb[:])
-                # If the nearest point is too close, resample.
+                # If the nearest point is too close, resample
                 while any([np.all(np.abs(x_new - xj) < self.eps)
                            for xj in self.x_vals]):
                     x_new[:] = self.lb[:] + (np.random.random(self.n)
